#include "bsp_soft_i2c.h"
#include "bsp_uart.h"
#include "log.h"

#undef LOG_ENABLE
#define LOG_ENABLE 1

#define LOG_LVL    LOG_DEBUG

#undef LOG_TAG
#define LOG_TAG      "i2c"



//初始化IIC
void IIC_Init(void)
{
    GPIOB_ModeCfg(IIC_SCL_BPIN | IIC_SDA_BPIN, GPIO_ModeOut_PP_5mA);

    IIC_SCL_H();
    IIC_SDA_H();
}

void IIC_Deinit(void)
{
    GPIOB_ModeCfg(IIC_SCL_BPIN | IIC_SDA_BPIN, GPIO_ModeIN_PU);
}

// 产生IIC起始信号
void IIC_Start(void)
{
    IIC_SDA_OUT();     //sda线输出

    IIC_SDA_H();
    IIC_SCL_H();
    DelayUs(4);
    IIC_SDA_L(); // START: when CLK is high, DATA changes from high to low
    DelayUs(4);
    IIC_SCL_L(); //钳住I2C总线，准备发送或接收数据
}

//产生IIC停止信号
void IIC_Stop(void)
{
    IIC_SDA_OUT(); //sda线输出

    IIC_SCL_L();
    IIC_SDA_L(); //STOP: when CLK is high DATA changes from low to high
    DelayUs(4);
    IIC_SCL_H();
    IIC_SDA_H(); //发送I2C总线结束信号
    DelayUs(4);
}

//等待应答信号到来
//返回值：1，接收应答失败
//        0，接收应答成功
uint8_t IIC_Wait_Ack(void)
{
    uint8_t ucErrTime = 0;
    IIC_SDA_IN();      //SDA设置为输入
    IIC_SDA_H(); DelayUs(1);
    IIC_SCL_H(); DelayUs(1);
    while (IIC_READ_SDA())
    {
        ucErrTime++;
        if (ucErrTime > 250)
        {
            IIC_Stop();
            return 1;
        }
    }
    IIC_SCL_L(); //时钟输出0
    return 0;
}

//产生ACK应答
void IIC_Ack(void)
{
    IIC_SCL_L();
    IIC_SDA_OUT();
    IIC_SDA_L();
    DelayUs(2);
    IIC_SCL_H();
    DelayUs(2);
    IIC_SCL_L();
}

//不产生ACK应答
void IIC_NAck(void)
{
    IIC_SCL_L();
    IIC_SDA_OUT();
    IIC_SDA_H();
    DelayUs(2);
    IIC_SCL_H();
    DelayUs(2);
    IIC_SCL_L();
}

//IIC发送一个字节
//返回从机有无应答
//1，有应答
//0，无应答
void IIC_Send_Byte(uint8_t txd)
{
    uint8_t t;
    IIC_SDA_OUT();
    IIC_SCL_L(); //拉低时钟开始数据传输
    for (t = 0; t < 8; t++)
    {
        ((txd & 0x80) >> 7) == 0 ? IIC_SDA_L() : IIC_SDA_H();
        txd <<= 1;
        DelayUs(2);   //对TEA5767这三个延时都是必须的
        IIC_SCL_H();
        DelayUs(2);
        IIC_SCL_L();
        DelayUs(2);
    }
}

//读1个字节，ack=1时，发送ACK，ack=0，发送nACK
uint8_t IIC_Read_Byte(unsigned char ack)
{
    unsigned char i, receive = 0;
    IIC_SDA_IN(); //SDA设置为输入
    for (i = 0; i < 8; i++)
    {
        IIC_SCL_L();
        DelayUs(2);
        IIC_SCL_H();
        receive <<= 1;
        if (IIC_READ_SDA()) receive++;
        DelayUs(1);
    }
    if (!ack)
        IIC_NAck(); //发送nACK
    else
        IIC_Ack(); //发送ACK
    return receive;
}

